#include "atari.h"
#include "config.h"
#include "monitor.h"
#include "colours.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "atari_gp32.h"
#include "colour_hsv.h"
#include "gp2xminlib.h"

//#define COUNT_FPS
#ifdef COUNT_FPS
#define DEBUG_TEXT
#endif
//#define DEBUG_TEXT
#define PALETTE

// Used externally as an easy way of getting text on the screen!
char * debugTextBuffer = 0;
char * debugText =0;

// Brightness (used externally in state save)
char brightness = 2;

// Screen buffers -> theoretically we double buffer.
// In practice it is not bad without so it is off ATM
// I have performance concerns with GpSwapBuffer
// Need to investigate further
// Update: Just tried -> performance reduced by approx 30-40%
// WTF? Guess it must be waiting for sync?
int pal[256];

// Interrupt driven frame count
// Every second we get called back and store the value here
#ifdef COUNT_FPS
static char countFPSResult[256];
static int drawFPS = 0;
static int realFPS = 0;
static int realFPSCount = 0;
static int drawFPSCount = 0;
void fpsCallback() { drawFPS = drawFPSCount; drawFPSCount = 0; realFPS = realFPSCount; realFPSCount = 0; }
void countFPS(int fullRender);
#endif

// Notes on DIRTYRECT
// Tried it and found it to be slower than without.
// Seemed to be that when it was enabled the antic rendering took 10ms longer. Since the copy to screen takes 5ms that isn't a gain.

#ifndef PI
#define PI 3.142
#endif

///////////////////////////////////////////////////////////////
// Interface, called externally
void * Atari_DisplayActiveBuffer()
{
#ifdef PALETTE
	return gp2x_screen8;
#else
	return gp2x_screen15;
#endif
}

void Atari_DisplayInitPhase2()
{
#ifdef PALETTE
	//gp2Draw = SDL_SetVideoMode(320,240,8,SDL_HWSURFACE|SDL_HWPALETTE);
	gp2x_video_init(8);
#else
	gp2x_video_init(16);
#endif
	Atari_DisplayBrightness(brightness);

	gp2x_fill_rect(0,0,320,240,0);
	gp2x_video_flip();
}

void Atari_DisplayInitialise()
{
#ifdef DEBUG_TEXT
	debugTextBuffer = (char *) malloc(256);
#endif

	//// Set palette (colortable is setup by emulator palette code)
	//Atari_DisplayBrightness(brightness);

#ifdef COUNT_FPS
	// FPS count
	GpTimerOptSet(gpNextTimer, 1, 0, fpsCallback);
	GpTimerSet(gpNextTimer++);
#endif
}

void Atari_DisplayBrightness(int level)
{
	//SDL_Color entry[256];
	int i;

	brightness = level;
	int sinScaleLevel = level&7;
	int fluLevel = level&8;

	for( i = 0 ; i < 256 ; i++ )
	{
		int R = (colortable[i]&0xff0000) >> 16;
		int G = (colortable[i]&0xff00) >> 8;
		int B = (colortable[i]&0xff) >> 0;

		double sinScale = 256;
		switch (sinScaleLevel)
		{
		case 1:
			sinScale = 256;
			break;
		case 2:
			sinScale = 320;
			break;
		case 3:
			sinScale = 380;
			break;
		case 4:
			sinScale = 500;
			break;
		}

		double Rd=R,Gd=G,Bd=B;

		if (fluLevel)
		{
			double H,S,V;
			rgb2hsv(Rd,Gd,Bd,&H,&S,&V);
			double Vold = V;
			if (Vold < 0.5)
			{
				V*=1.1;
			}
			if (Vold < 0.3)
			{
				V*=1.1;
			}
			if (Vold < 0.1)
			{
				V*=1.1;
			}
			hsv2rgb(H,S,V,&Rd,&Gd,&Bd);
		} 
		// FLU can be a bit washed out -> increase the saturation

		if (sinScaleLevel != 0)
		{
			Rd = PI*(double)Rd/(double)512;
			Gd = PI*(double)Gd/(double)512;
			Bd = PI*(double)Bd/(double)512;

			Rd = sin(Rd)*sinScale;
			Gd = sin(Gd)*sinScale;
			Bd = sin(Bd)*sinScale;
		}
		// Since GP32 is so fscking dull, need to boost mid range
		// Don't really want to boost dark stuff since I like true black
		// So use a nice curve

		if (fluLevel || sinScaleLevel)
		{
			R = Rd<256 ? Rd : 255;
			G = Gd<256 ? Gd : 255;
			B = Bd<256 ? Bd : 255;
		}

#ifdef PALETTE
		/*entry[i].unused = 0;
		entry[i].r = R;
		entry[i].g = G;
		entry[i].b = B;*/
#endif
		pal[i] = (R<<16)|(G<<8)|B;
	}

	for (i=0; i<256; ++i)
	{
#ifdef PALETTE
		gp2x_video_color8(i,(pal[i]&0xFF0000)>>16, (pal[i]&0xFF00)>>8, (pal[i]&0xFF)); 
#endif
		pal[i] = gp2x_video_color15((pal[i]&0xFF0000)>>16, (pal[i]&0xFF00)>>8, (pal[i]&0xFF), 0);
	}

#ifdef PALETTE
	//SDL_SetPalette(gp2Draw, SDL_PHYSPAL, entry, 0, 256);
	gp2x_video_setpalette();
#endif
}

void Atari_DisplayScreen(UBYTE * screen)
{	
	int i,j;

	// Passed a null pointer if we are not expected to draw
	// Called every frame regardless...
#ifdef DEBUG_TEXT
	int oldbackbuf = backbuf;
#endif

	if (screen)
	{
	//	if (SDL_MUSTLOCK(gp2Draw))
	//		SDL_LockSurface(gp2Draw);
		
		int width = 320-menuWidth;
		int offset = menuWidth + ((ATARI_WIDTH-320)/2);
		int height = 240-menuHeight;

#ifdef PALETTE
		char * dest = ((char *) gp2x_screen8) + menuWidth;
#else
		unsigned int * dest = (unsigned int *)(((short *) gp2x_screen15) + menuWidth);
#endif
		unsigned char * src = (unsigned char *) (screen + offset);

		int y=0;
		for (y=0; y<height; ++y)
		{
#ifdef PALETTE
			memcpy(dest,src,width);
			src+=ATARI_WIDTH;
			dest+=320;
#else
			int x=0;
			for (x=0; x<width; x+=2)
			{
				dest[x>>1] = pal[src[x+1]]<<16|pal[src[x]]; 
			}
			src += ATARI_WIDTH;
			dest += 160;
#endif
		}

		//if (SDL_MUSTLOCK(gp2Draw))
		//	SDL_UnlockSurface(gp2Draw);
		//SDL_UpdateRect(gp2Draw,0,0,0,0);
		gp2x_video_flip();
	}

//	debugText = debugTextBuffer;
//	sprintf(debugTextBuffer, "RTC:%8d %8d\n", rtcTick, GpTickCountGet());

#ifdef DEBUG_TEXT
	if (debugText)
	{
		GpRectFill(NULL, &gpDraw[oldbackbuf],5,220,305,10, 0x00);
		GpTextOut(0, &gpDraw[oldbackbuf], 9, 220, debugText, 0xff);
	}
#endif

#ifdef COUNT_FPS
	countFPS(screen!=0);
	GpRectFill(NULL, &gpDraw[oldbackbuf],5,230,305,10, 0x00);
	GpTextOut(0, &gpDraw[oldbackbuf], 9, 230, countFPSResult, 0xff);
#endif
}

///////////////////////////////////////////////////////////////
// Internal
#ifdef COUNT_FPS
void countFPS(int fullRender)
{
	static int first = 1;
	static int lastTicks = 0;
	static int totalTicks = 0;
	static int totalFrames = 0;
	static int skipCount = 0;

	static int fullRenderTicks[10];
	static int fullRenderPos = 0;

	if (first)
	{
		lastTicks = GpTickCountGet();
		first = 0;
	}
	else
	{
		int now = GpTickCountGet();
		int taken = now - lastTicks;
		int avgFrames = ((++totalFrames)*1000)/totalTicks;

		lastTicks = now;
		totalTicks += taken;

		if (fullRender) // Did we render this frame?
		{
			fullRenderPos++;
			if (fullRenderPos == 10)
				fullRenderPos = 0;

			fullRenderTicks[fullRenderPos] = taken;

			drawFPSCount++;
		}
		else
			skipCount++;
		realFPSCount++;

		int fullRenderTickCount=0;
		int k;
		for (k=0;k<10;++k)
			fullRenderTickCount += fullRenderTicks[k];

		sprintf(countFPSResult, "dps:%2d eps:%2d skp:%d rms:%2d rfp:%2d", drawFPS, realFPS, skipCount, fullRenderTickCount/10, 10000/fullRenderTickCount);

	}
}
#endif

int mapPal(int input)
{
#ifdef PALETTE
	return input;
#else
	return pal[input];
#endif
}

