#include <stdlib.h>
#include <sched.h>

#include "atari.h"
#include "config.h"
#include "monitor.h"
#include "colours.h"
#include "pokeysnd.h"
#include "input.h"

#include "sound.h"
#include "platform.h"
#include "antic.h"
#include "gtia.h"
#include "pokey.h"
#include "diskled.h"
#include "devices.h"
#include "log.h"
#include "pia.h"
#include "ui.h"
#include "rt-config.h"
#include "statesav.h"

#include "atari_gp32.h"
#include "splash.h"

#include "menu_keymap.h"
#include "menu_c.h"

#include "gp2xminlib.h"

#include "cpu_speed.h"

#include <stdio.h>
#include <string.h>

#include <sys/stat.h>

void dumpTimerRegs();
int gpNextTimer = 0;

#ifdef NEW_CYCLE_EXACT
int gp32ClockEnum = GPMHZ_200;
static int gp32ClockEnumMax = GPMHZ_250;
#else
int gp32ClockEnum = GPMHZ_200;
static int gp32ClockEnumMax = GPMHZ_250;
#endif
extern int fastForward;

int main(int argc, char * argv[])
{
	struct sched_param param;
	param.sched_priority=50;

	//sched_setscheduler(0, SCHED_FIFO, &param);
	nice(-10);

	SDL_Init(SDL_INIT_TIMER|SDL_INIT_NOPARACHUTE);
	SDL_ShowCursor(0);

	gp2x_reg_init();
	initSpeed();

/*	SDL_Surface * s = SDL_SetVideoMode(320,240,32,SDL_HWSURFACE);
	SDL_Rect rect;
	rect.x = 5;
	rect.y = 5;
	rect.w = 100;
	rect.h = 100;

	SDL_FillRect(s, &rect, 0xff);
	SDL_UpdateRect(s,0,0,0,0);
	for (;;);
*/	
	// Display splash screen
	initSplash();
	
	// Initialise emulator (this will call us back in Atari_Initialise)
	if (!Atari800_Initialise(&argc, argv))
	{
		Atari_Exit(0);
		return 0;
	}

	// Start in ui so user can load a game - unless autorun exists
#ifndef NATIVE
	char statename[FILENAME_MAX+1];
	strcpy(statename, atari_state_dir);
	strcat(statename, "\\autorun");

	struct stat statBuf;
	int loadedState =0;
	if (!stat(statename, &statBuf))
		loadedState = ReadAtariState(statename, "rb");
#else
	int loadedState = 0;
#endif

	cleanupSplash(); 
	Atari_DisplayInitPhase2();
	
	// If autorun exists load it, else go to ui
	if (!loadedState)
		ui((UBYTE *) atari_screen);

	Sound_Continue();

	// Main loop

	int fastForwardFrameCount = 0;
	while (key_code != AKEY_EXIT) {
#ifdef TIMESECTIONS
		int frameStart = GpTickCountGet();
#endif

		INPUT_Frame();
			
#ifdef TIMESECTIONS
		int inputFrame = GpTickCountGet();
#endif

		Device_Frame();
#ifdef TIMESECTIONS
		int deviceFrame = GpTickCountGet();	
#endif

		GTIA_Frame();
#ifdef TIMESECTIONS
		int gtiaFrame = GpTickCountGet();	
#endif

		if (!fastForward)
		{
			Sound_Update();
		}
#ifdef TIMESECTIONS
		int soundFrame = GpTickCountGet();	
#endif

		//timeDrawNextFrame = 1;
		if (fastForward)
		{
			fastForwardFrameCount++;
			fastForwardFrameCount&=0xf;
			timeDrawNextFrame = (fastForwardFrameCount==0);
		}

		ANTIC_Frame(timeDrawNextFrame);
#ifdef TIMESECTIONS
		int anticFrame = GpTickCountGet();	
#endif

		POKEY_Frame();
#ifdef TIMESECTIONS
		int pokeyFrame = GpTickCountGet();	
#endif

		Menu_Frame();
#ifdef TIMESECTIONS
		int menuFrame = GpTickCountGet();
#endif

		Atari_DisplayScreen(timeDrawNextFrame ? (UBYTE *) atari_screen : 0);
#ifdef TIMESECTIONS
		int displayFrame = GpTickCountGet();	
#endif

#ifdef TIMESECTIONS
		char arse[256];
		debugText = arse;
		sprintf(arse, "D%3d G%3d S:%3d A:%3d P:%3d A:%3d", deviceFrame-frameStart, gtiaFrame-deviceFrame, soundFrame-gtiaFrame, anticFrame-soundFrame, pokeyFrame-anticFrame, displayFrame-pokeyFrame);
#endif

		nframes++;

		/*if (timeDrawNextFrame)
		{
			GpSurfaceFlip(&gpDraw[backbuf]);
			backbuf ^= 1;
		}*/

		//atari_sync();
		if (!fastForward)
			Atari_Sync();
	}
	Atari800_Exit(FALSE);
	Aflushlog();
	return 0;
}

void Atari_Initialise(int *argc, char *argv[])
{
	Atari_DisplayInitialise();
	Keyboard_Initialise();
	Sound_Initialise(0,0);

	// Menu
	Menu_Initialise();
}

int Atari_Exit(int run_monitor)
{
	// FIXME -> display exit screen or something
	replaceSpeed();
	gp2x_deinit();
	
	return FALSE;
}

static char platform[] = "GP32";

void PlatformStateSave()
{
	SaveUBYTE(platform, strlen(platform)); 

	// Brightness
	SaveUBYTE(&brightness, 1);

	// Menu
	SaveUBYTE(currentMenu, strlen(currentMenu)+1);

	// Frame skip
	unsigned char frameSkip = Atari_GetAutoFrameSkip();
	SaveUBYTE(&frameSkip, 1);

	// Artifacting
	unsigned char artifMode = global_artif_mode;
	SaveUBYTE(&artifMode, 1);
	
	// Clock speed
	SaveUBYTE(&gp32ClockEnum, 1);
}

void PlatformStateRead()
{
	char temp[FILENAME_MAX];

	strcpy(temp, platform);
	temp[0] = '\0';
	ReadUBYTE(temp, strlen(platform));
	if (strcmp(temp, platform) != 0)
		return;

	// Brightness
	ReadUBYTE(&brightness, 1);
	Atari_DisplayBrightness(brightness);

	// Menu
	char menuTemp = 1;
	int iter = 0;
	do
	{
		ReadUBYTE(&menuTemp, 1);
		temp[iter++] = menuTemp;
	} while (menuTemp!=0);

	menuLoad(temp);

	// Frame skip
	unsigned char frameSkip = 0xff;
	ReadUBYTE(&frameSkip, 1);
	if (frameSkip != 0xff)
		Atari_AutoFrameSkip(frameSkip);

	// Artifacting
	unsigned char artifMode = 0xff;
	ReadUBYTE(&artifMode, 1);
	if (artifMode != 0xff)
	{	
		global_artif_mode = artifMode;
		ANTIC_UpdateArtifacting();
	}

	// Clock speed
	unsigned char lgp32ClockEnum = 0xff;
	ReadUBYTE(&lgp32ClockEnum, 1);
	if (lgp32ClockEnum != 0xff)
		GP32_SetClock(lgp32ClockEnum);
}

void GP32_SetClockMax(int onOff)
{
	if (onOff)
	{
		int gp32ClockEnumStore = gp32ClockEnum;
		GP32_SetClock(gp32ClockEnumMax);
		gp32ClockEnum = gp32ClockEnumStore;
	}
	else
	{
		GP32_SetClock(gp32ClockEnum|0x80); //Force...
	}
}

void GP32_SetClock(int clock)
{
	int freq=0;
	int magic=0;
	int div=0;
	
	if (clock == gp32ClockEnum)
		return;
	clock &= 0x7f;
	gp32ClockEnum = clock;
	if (clock > gp32ClockEnumMax)
		gp32ClockEnumMax = clock;

	switch(clock)
	{
//	case GPMHZ_40: GpClockSpeedChange (40000000, 0x48013, 1); break;
//	case GPMHZ_59: GpClockSpeedChange (59250000, 0x47022, 1); break;
	case GPMHZ_67: freq=66; div=0; break; 
	case GPMHZ_80: freq=80; div=0; break;
	case GPMHZ_99: freq=100; div=0; break;
	case GPMHZ_115: freq=120; div=0; break; 
	case GPMHZ_132: freq=133; div=0; break;
	case GPMHZ_156: freq=150; div=0; break;
	case GPMHZ_165: freq=166; div=0; break;
	case GPMHZ_180: freq=180; div=0; break;
	case GPMHZ_200: freq=200; div=0; break;
	case GPMHZ_250: freq=250; div=0; break;
	default:exit(0);
	}

	setSpeed(freq, div );

/*	GpClockSpeedChange(freq, magic, div);
	Aprint("Clock M:%d H:%d P:%d\n", GpMClkGet(), GpHClkGet(), GpPClkGet());
	unsigned int pclk = GpPClkGet();

	// Repair SDK timer - it forgets to set prescaler
	//pclk/(x+1) ~= 8000*40
	//x= (pclk/(8000*40)) -1
	unsigned int prescaler0 = (pclk/(8000*40))-1;
	rTCFG0 = (rTCFG0&0xFFFFFF00)|prescaler0;
	rTCFG1 = 0x30033;

	// Repair GpTickCountGet
	// pclk/(100x) = 16
	// x = pclk/1600
	rTCNTB4 = pclk/1600;
 //	rTCON  = (0x1<<22) | (0x1<<20);

	// Repair display - driven off hclk
	// HMM - looks like the sdk doesn't tweak clkval and I can't find anything else. Worst case we'll get < 2x lcd framerate (49.. vs 80 hck)

	// Repair sound
#ifdef STEREO
	GpPcmInit(PCM_S22, PCM_8BIT);
#else
	GpPcmInit(PCM_M22, PCM_8BIT);
#endif

	// Repair chatboard
	preInitKbd(9600, GpPClkGet());

	dumpTimerRegs();
*/
//	if (div>1)
//		div = 2<<(div-2);
//	clockSpeed = freq/ div;
}



