#include "atari.h"
#include "config.h"
#include "input.h"
#include "ui.h"
#include "atari.h"
#include "rt-config.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "atari_gp32.h"
#include "sound.h"
#include "menu_c.h"
#include "menu_keymap.h"
#include "gp2xjoy.h"

#include "log.h"

unsigned char menuWidth = 0; //Either height or width at present...
unsigned char menuHeight = 0;
int fastForward = 0;

void setFastForward(int onOff)
{
	if (onOff == fastForward)
		return;
	if (onOff)
	{
		// Set to max known possible clock speed
		Sound_Pause();
		GP32_SetClockMax(1);
		fastForward = 1;
	}
	else
	{
		GP32_SetClockMax(0);
		Sound_Continue();
		fastForward = 0;
	}
}

///////////////////////////////////////////////////////////////////////
// Interface -> called externally
void Menu_Initialise()
{
	initKeymap();

	char * menuFile = malloc(FILENAME_MAX);

	strcpy(menuFile, atari_menu_dir);
	strcat(menuFile, "/ui.men");
	
	menuActivateUi();
	menuLoad(menuFile);

	strcpy(menuFile, atari_menu_dir);
	strcat(menuFile, "/default.men");
	
	menuActivateMain();
	menuLoad(menuFile);

	free(menuFile);

	// Init chatboard listener
	//TODO preInitKbd(9600, GpPClkGet());
}

void Menu_Frame()
{
	int x,y,buttons;
	readAllJoy(&x,&y,&buttons);
	menuFrame(x,y,buttons);

	int operation = menuOper();
	key_code = menuKey();
	key_consol = menuConsol();
	key_shift = menuShift();
	menuJoy(joyValue, fireValue);

	// Override key with chatboard for now!
	key_break = 0;
	static int ctrl = 0;
	int ch = -1; //TODO (chatboard)pollKbdKey();
	if (ch >=0)
	{
//		debugText = debugTextBuffer;
//		sprintf(debugTextBuffer,"Key: %d\n", key_code);

		if (ch == 0x7f)
			key_break = 1;
		else if (ch == 0x7e)
			ctrl = 1;
		else
		{
			key_code = ch;

			if (ctrl)
			{
				ctrl = 0;
				key_code |= AKEY_CTRL;
			}
		}
	}	
	else if (key_code == AKEY_BREAK)
	{
		key_break = 1;
	}

	switch(operation)
	{
	case AKEY_FASTFORWARDON:
		setFastForward(1);
		break;
	case AKEY_FASTFORWARDOFF:
		setFastForward(0);
		break;
	case AKEY_FASTFORWARDTOGGLE:
		setFastForward(!fastForward);
		while (joyPressed());
		break;
	case AKEY_COLDSTART:
		menuClean();
		Coldstart();
		break;
	case AKEY_WARMSTART:
		menuClean();
		Aprint("Warmstart\n");
		Warmstart();
		Aprint("Warmstart done\n");
		break;

	case  AKEY_UI:
		if (fastForward) //Hack...
		{
			setFastForward(0);
			break;
		}
		setFastForward(0);
		Sound_Pause();
		menuWidth = 0;
		menuHeight = 0;
		ui((UBYTE *) atari_screen);
		while (joyPressed());
		menuClean();
		menuWidth = 0;
		menuHeight = 0;
		Sound_Continue();
		break;
	}
}

