#include "atari.h"
#include "config.h"
#include "monitor.h"
#include "colours.h"

#include <math.h>
#include <stdlib.h>

#include "atari_gp32.h"

// Variable may be used to see difference between GP32 clock and Atari clock
static int timeDiff = 0;
int timeDrawNextFrame = 0;

// Frame skip enabled?
static int autoFrameSkip = 1; //automatic enabled by default

static void atariSync(int resync)
{
	static int frameCount = 0;
	static int startTime = 0xbadf00d;

	int tickDeltaTime = deltatime*1000.0;

	// Calulate where we are now
	int runTime = Atari_Time() - startTime;
	frameCount++;

	// Deltatime is 1/50 for PAL, 1/60 for NTSC
	double expectedTime = tickDeltaTime * frameCount;

	// Store diff
	timeDiff = (runTime-expectedTime);

	// resync if miles off!
	double diff = abs(timeDiff);
	if (resync || (diff > (5*tickDeltaTime))) // More than 5 frames off
	{
		runTime = expectedTime;
		startTime = Atari_Time() - runTime;
		timeDrawNextFrame = !autoFrameSkip;
		return;
	}

	//Are we ahead of where we should be?
	if (timeDiff < 0)
	{
		timeDrawNextFrame = 1;

		// Are we way too fast? Don't want to be over half a frame ahead -> if we are then wait some
		
		double requiredTime = expectedTime - ((9*tickDeltaTime)/10);
//		sprintf(debugTextBuffer,"Wait: real:%d req:%d diff:%d", Atari_Time()-startTime, (int)requiredTime, (int)requiredTime-Atari_Time()+startTime);
//		debugText = debugTextBuffer;
		while ((Atari_Time()-startTime) < requiredTime) usleep(1000); 
	}
	else
	{
		timeDrawNextFrame = !autoFrameSkip;
	}
}

////////////////////////////////////////////////////////////////////////////
// Interface -> called externally
int Atari_Time()
{
	return GpTickCountGet();
//	return rtcTick*1000/64;
}

void Atari_Sync()
{
	atariSync(0);
}

void Atari_ReSync()
{
	atariSync(1);
}

void Atari_AutoFrameSkip(int enable)
{
	autoFrameSkip = enable;
}

int Atari_GetAutoFrameSkip()
{
	return autoFrameSkip;
}

void atari_sync() { return Atari_Sync(); } 



