extern "C"
{
#include "menu_c.h"
#include "input.h"
#include <stdio.h>
#include <string.h>
#include "atari_gp32.h"
}

#include "menu_container.h"
#include "menu_amenu.h"

char currentMenu[FILENAME_MAX];

static MenuContainer ** container = 0;
static MenuContainer * uiContainer = 0;
static MenuContainer * mainContainer = 0;

void menuActivateUi()
{
	container = &uiContainer;
}

void menuActivateMain()
{
	container = &mainContainer;
}

void menuLoad(const char * filename)
{
	if (*container)
		delete (*container);
	(*container) = new MenuContainer;

	(*container)->loadMenu(filename);
	menuDump();
	printf("Filename:%s\n", filename);
	if (container == &mainContainer)
		strcpy(currentMenu, filename);
}	

void menuFrame(int x, int y, int buttons)
{
	if (!(*container))
		return;

	// convert buttons to gp32 stylee
	switch (x)
	{
	case -1:
		x=1;
		break;
	case 1:
		x=2;
		break;
	}
	switch (y)
	{
	case -1:
		y=1;
		break;
	case 1:
		y=2;
		break;
	}
	int keysPressed = buttons | (y<<8) | (x<<10);

	menuWidth = 0;
	menuHeight = 0;

	MenuContainer::const_iterator i;
	for (i=(*container)->begin(); i!=(*container)->end(); ++i)
	{
		AMenu * amenu = *i;
		if (amenu==(*container)->d_blocking || !(*container)->d_blocking || !amenu->mayBeBlocked())
		{
			amenu->frame(keysPressed);
			if (amenu->blockOthers())
				(*container)->d_blocking = amenu;
		}
		else
			amenu->clean();
	}

	if ((*container)->d_blocking && !(*container)->d_blocking->blockOthers())
		(*container)->d_blocking = 0;
}

void menuClean()
{
	if (!(*container))
		return;

	MenuContainer::const_iterator i;
	for (i=(*container)->begin(); i!=(*container)->end(); ++i)
	{
		AMenu * amenu = *i;
		amenu->clean();
	}

	(*container)->d_blocking = 0;
}

int menuOper()
{
	if (!(*container))
		return 0;

	int oper = 0;
	if ((*container)->d_blocking)
		oper = (*container)->d_blocking->oper();

	MenuContainer::const_iterator i;
	for (i=(*container)->begin(); i!=(*container)->end(); ++i)
	{
		AMenu * amenu = *i;
		if ((*container)->d_blocking)
		{
			if ((*container)->d_blocking!=amenu && !amenu->mayBeBlocked())
			{
				int tempOper = amenu->oper();
				if (tempOper!=0)
					oper = tempOper;
			}
		}
		else
		{
			int tempOper = amenu->oper();
			if (tempOper!=0)
				oper = tempOper;
		}
	}

	return oper;
}

int menuKey()
{
	if (!(*container))
		return AKEY_NONE;

	int key = AKEY_NONE;
	if ((*container)->d_blocking)
		key = (*container)->d_blocking->key();

	MenuContainer::const_iterator i;
	for (i=(*container)->begin(); i!=(*container)->end(); ++i)
	{
		AMenu * amenu = *i;
		if ((*container)->d_blocking)
		{
			if ((*container)->d_blocking!=amenu && !amenu->mayBeBlocked())
			{
				int tempKey = amenu->key();
				if (tempKey!=AKEY_NONE)
					key = tempKey;
			}
		}
		else
		{
			int tempKey = amenu->key();
			if (tempKey!=AKEY_NONE)
				key = tempKey;
		}
	}

	return key;
}

int menuConsol()
{
	if (!(*container))
		return CONSOL_NONE;

	int consol = CONSOL_NONE;
	if ((*container)->d_blocking)
		consol &= ~(*container)->d_blocking->consol();

	MenuContainer::const_iterator i;
	for (i=(*container)->begin(); i!=(*container)->end(); ++i)
	{
		AMenu * amenu = *i;
		if ((*container)->d_blocking)
		{
			if ((*container)->d_blocking!=amenu && !amenu->mayBeBlocked())
			{
				int tempConsol = amenu->consol();
				if (tempConsol!=CONSOL_NONE)
					consol &= ~tempConsol;
			}
		}
		else
		{
			int tempConsol = amenu->consol();
			if (tempConsol!=CONSOL_NONE)
				consol &= ~tempConsol;
		}
	}

	return consol;
}

int menuShift()
{
	if (!(*container))
		return 0;

	bool shift = false;

	MenuContainer::const_iterator i;
	for (i=(*container)->begin(); i!=(*container)->end(); ++i)
	{
		AMenu * amenu = *i;
		shift |= amenu->shift();
	}

	return shift;
}

void menuJoy(int stick[], int fire[])
{
	int count = 4;
	int j = 0;

	// Init to defaults
	for (j=0; j<count; ++j)
	{
		stick[j] = 0xff;
		fire[j] = 1;
	}	

	// Ignore if no container!
	if (!(*container) || (*container)->d_blocking)
		return;

	// Loop through combining key values
	MenuContainer::const_iterator i;
	for (i=(*container)->begin(); i!=(*container)->end(); ++i)
	{
		AMenu * amenu = *i;
		int joy = amenu->joy();

		if (joy&1)
			fire[0] = 0;
		if (joy&(1<<5))
			fire[1] = 0;

		stick[0]&=~(0xf&(joy>>1));
		stick[1]&=~(0xf&(joy>>6));
	}

	return;
}

void menuDump()
{
	if (!(*container))
		return;

	MenuContainer::const_iterator i;
	for (i=(*container)->begin(); i!=(*container)->end(); ++i)
	{
		AMenu * amenu = *i;
		printf("Type: %s Driver: %x\n", amenu->type().c_str(), amenu->driver());
	}
}

